/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

struct post *
info (int id)
{
  struct MemoryStruct *chunk;
  struct post *tmp;
  char file[SIZE_BUFFER];

  snprintf (file, SIZE_BUFFER, "%s?id_request=%d", xmlpath, id);

  if (debug)
    fprintf (stderr, "Info search file: %s\n", file);

  if (!(chunk = download_file (file)))
    return NULL;

  if (!(tmp = info_check (chunk->mm, chunk->size)))
    return NULL;

  free (chunk->mm);
  free (chunk);

  return tmp;
}

struct post *
info_check (void *mm, size_t size)
{
  struct post *tmp = NULL;

  xmlDocPtr doc;
  xmlNodePtr cur;

  if (!(doc = xmlParseMemory (mm, size)))
    return NULL;

  if (!(cur = xmlDocGetRootElement (doc)))
    return NULL;

  if (xmlStrcmp (cur->name, (const xmlChar *) "org"))
    {
      xmlFreeDoc (doc);
      return NULL;
    }

  cur = cur->xmlChildrenNode;

  if (!(tmp = (struct post *) malloc (sizeof (struct post))))
    fatal ("Error of memory.");

  if (!cur->next)
    {
      free (tmp);
      return NULL;
    }
  cur = cur->next;

  tmp->id = xmlGetProp (cur, "id") ? atoi (xmlGetProp (cur, "id")) : 0;
  tmp->date = xmlGetProp (cur, "date_entered");
  tmp->link = xmlGetProp (cur, "linked_file");
  tmp->url = xmlGetProp (cur, "url");
  tmp->mime = xmlGetProp (cur, "mime_type");
  tmp->comments =
    xmlGetProp (cur,
		"numcomment") ? atoi (xmlGetProp (cur, "numcomment")) : 0;
  tmp->author = xmlGetProp (cur, "author");

  cur = cur->xmlChildrenNode;
  if (!cur || !cur->next)
    {
      free (tmp);
      return NULL;
    }
  cur = cur->next;

  tmp->title = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (!cur->next || !cur->next->next)
    {
      free (tmp);
      return NULL;
    }
  cur = cur->next->next;

  tmp->summary = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  xmlFreeDoc (doc);

  return tmp;
}

void
info_write (char *who, struct post *tmp)
{
  char *t;
  int len;
  char output[SIZE_BUFFER];

#ifdef HAVE_LIBRECODE
  t = recode (tmp->date);
#else
  t = tmp->date;
#endif

  len =
    snprintf (output, SIZE_BUFFER, "PRIVMSG %s :Id: %d - date: %s\n", who,
	      tmp->id, t);
  irc_write (output, len);

#ifdef HAVE_LIBRECODE
  free (t);
#endif

  if (tmp->link)
    {

#ifdef HAVE_LIBRECODE
      t = recode (tmp->link);
#else
      t = tmp->link;
#endif

      len =
	snprintf (output, SIZE_BUFFER, "PRIVMSG %s :Linked File: %s\n", who,
		  t);
      irc_write (output, len);

#ifdef HAVE_LIBRECODE
      free (t);
#endif

    }

  if (tmp->url)
    {

#ifdef HAVE_LIBRECODE
      t = recode (tmp->url);
#else
      t = tmp->url;
#endif

      len = snprintf (output, SIZE_BUFFER, "PRIVMSG %s :url: %s\n", who, t);
      irc_write (output, len);

#ifdef HAVE_LIBRECODE
      free (t);
#endif

    }

  len =
    snprintf (output, SIZE_BUFFER, "PRIVMSG %s :comments: %d\n", who,
	      tmp->comments);
  irc_write (output, len);

  if (tmp->author)
    {
#ifdef HAVE_LIBRECODE
      t = recode (tmp->author);
#else
      t = tmp->author;
#endif

      len =
	snprintf (output, SIZE_BUFFER, "PRIVMSG %s :author: %s\n", who, t);
      irc_write (output, len);

#ifdef HAVE_LIBRECODE
      free (t);
#endif

    }

  if (tmp->title)
    {
#ifdef HAVE_LIBRECODE
      t = recode (tmp->title);
#else
      t = tmp->title;
#endif

      len = snprintf (output, SIZE_BUFFER, "PRIVMSG %s :title: %s\n", who, t);
      irc_write (output, len);

#ifdef HAVE_LIBRECODE
      free (t);
#endif

    }

  if (tmp->summary)
    {
#ifdef HAVE_LIBRECODE
      t = recode (tmp->summary);
#else
      t = tmp->summary;
#endif

      len =
	snprintf (output, SIZE_BUFFER, "PRIVMSG %s :summary: %s\n", who, t);
      irc_write (output, len);

#ifdef HAVE_LIBRECODE
      free (t);
#endif

    }
}
