/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

int fd, check_time, check_write;
pthread_mutex_t mutex;

#ifdef HAVE_LIBSSL
BIO *bio;
#endif

char *file_cfg = NULL;
int debug = 0;

char *nick = NULL;
char *server = NULL;
char *password = NULL;
char *xmlpath = NULL;
int port;
int timer;
int secure;
int set_quit;

struct channel *channels = NULL;

#ifdef HAVE_LIBRECODE
RECODE_REQUEST request;
#endif

void signal_pipe(int);

int
main (int argc, char *argv[])
{
  cfg_t *cfg;
  register int i;

  static cfg_opt_t channel_opts[] = {
    CFG_STR ("File", 0, CFGF_NONE),
    CFG_END ()
  };

  static cfg_opt_t opts[] = {
    CFG_STR ("Nick", "indybot", CFGF_NONE),
    CFG_STR ("Server", NULL, CFGF_NONE),
    CFG_INT ("Port", 6667, CFGF_NONE),
    CFG_BOOL ("Ssl", cfg_false, CFGF_NONE),
    CFG_STR ("Password", NULL, CFGF_NONE),
    CFG_INT ("Timer", 1, CFGF_NONE),
    CFG_STR ("XmlPath", NULL, CFGF_NONE),
    CFG_SEC ("Channel", channel_opts, CFGF_MULTI | CFGF_TITLE),
    CFG_END ()
  };

#ifdef HAVE_LIBRECODE
  RECODE_OUTER outer = recode_new_outer (true);

  request = recode_new_request (outer);
  recode_scan_request (request, "utf8..latin1");
#endif

  signal (SIGPIPE, signal_pipe);

  for (i = 1; i < argc; i++)
    {
      if (!strcmp (argv[i], "-f"))
	{

	  i++;
	  if (argv[i])
	    file_cfg = argv[i];

	  else
	    fatal ("Error: argoment to '-f' is missing.");

	}
      else if (!strcmp (argv[i], "-d"))
	{
	  debug = 1;

	}
      else if (!strcmp (argv[i], "-h"))
	{
	  usage ();

	}
      else
	fatal ("Error: unrecognized option.");
    }

  if (!file_cfg)
    file_cfg = FILE_CFG;

  cfg = cfg_init (opts, CFGF_NOCASE);

  switch (cfg_parse (cfg, file_cfg))
    {

    case CFG_FILE_ERROR:
      fatal ("Error: no config files.");

    case CFG_PARSE_ERROR:
      fatal ("Error: file not recognized.");
    }

  if (!(nick = strdup (cfg_getstr (cfg, "Nick"))))
    fatal ("Error: memory.");

  if (!(server = strdup (cfg_getstr (cfg, "Server"))))
    fatal ("Error: memory.");

  if (!(xmlpath = strdup (cfg_getstr (cfg, "XmlPath"))))
    fatal ("Error: memory.");

  if (!nick || !*nick || !server)
    fatal ("Error: no nick or no server in config file.");

  if (!xmlpath)
    fatal ("Error: no xmlpath in config file.");

  if (strncmp (xmlpath, "http", 4))
    {
      char *a;

      if (!(a = (char *) malloc (sizeof (char) * (strlen (xmlpath) + 7))))
	fatal ("Error of memory.");
      sprintf (a, "http://%s", xmlpath);

      free (xmlpath);

      xmlpath = a;
    }

  port = cfg_getint (cfg, "Port");
  timer = cfg_getint (cfg, "Timer");
  secure = cfg_getbool (cfg, "Ssl");

#ifndef HAVE_LIBSSL
  if (secure)
    fatal ("No support for ssl. Compile with openssl.");
#endif

  if (!(password = strdup (cfg_getstr (cfg, "Password"))))
    fatal ("Error: memory.");

  if (!(i = cfg_size (cfg, "Channel")))
    fatal ("Error: no channel in config files.");

  while (i--)
    {
      cfg_t *pl;

      pl = cfg_getnsec (cfg, "Channel", i);

      add_channel (cfg_title (pl), cfg_getstr (pl, "File"));
    }

  cfg_free (cfg);

  if (!debug)
    {
      if (fork ())
	exit (0);

      setsid ();
    }

  while (!set_quit)
    {
      irc_socket ();

      sleep (1);
    }

#ifdef HAVE_LIBRECODE
  recode_delete_request (request);
  recode_delete_outer (outer);
#endif

  return 0;
}

char *
trim (char *tmp)
{
  int i = 0;
  while (tmp[i] == ' ' || tmp[i] == '\t' || tmp[i] == '\r' || tmp[i] == '\n')
    tmp++;

  i = strlen (tmp);
  i--;

  while (tmp[i] == ' ' || tmp[i] == '\t' || tmp[i] == '\r' || tmp[i] == '\n')
    i--;

  tmp[i + 1] = 0;

  return tmp;
}

void
fatal (const char *str)
{
  fprintf (stderr, "%s\n", str);

  exit (1);
}

/* SIGPIPE */
void signal_pipe(int signo) {
}

