/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

int
search_check (char *who, void *mm, size_t size)
{
  struct post *tmp = NULL;
  int i = 0;
  xmlDocPtr doc;
  xmlNodePtr cur, cur2;

  if (!(doc = xmlParseMemory (mm, size)))
    return 1;

  if (!(cur = xmlDocGetRootElement (doc)))
    return 1;

  if (xmlStrcmp (cur->name, (const xmlChar *) "org"))
    {
      xmlFreeDoc (doc);
      return 1;
    }

  cur = cur->xmlChildrenNode;

  while (cur)
    {

      if (!cur)
	{
	  free (tmp);
	  return 1;
	}

      if (!(tmp = (struct post *) malloc (sizeof (struct post))))
	fatal ("Error of memory.");

      if (!cur->next)
	{
	  free (tmp);
	  return 1;
	}

      cur = cur->next;

      tmp->id = xmlGetProp (cur, "id") ? atoi (xmlGetProp (cur, "id")) : 0;
      tmp->date = xmlGetProp (cur, "date_entered");
      tmp->link = xmlGetProp (cur, "linked_file");
      tmp->url = xmlGetProp (cur, "url");
      tmp->mime = xmlGetProp (cur, "mime_type");
      tmp->comments =
	xmlGetProp (cur,
		    "numcomment") ? atoi (xmlGetProp (cur, "numcomment")) : 0;
      tmp->author = xmlGetProp (cur, "author");

      cur2 = cur->xmlChildrenNode;
      if (!cur2 || !cur2->next)
	{
	  free (tmp);
	  return 1;
	}
      cur2 = cur2->next;

      tmp->title = xmlNodeListGetString (doc, cur2->xmlChildrenNode, 1);

      if (!cur2->next || !cur2->next->next)
	{
	  free (tmp);
	  return 1;
	}
      cur2 = cur2->next->next;

      tmp->summary = xmlNodeListGetString (doc, cur2->xmlChildrenNode, 1);

      search_write (who, tmp, &i);

      free (tmp);

      cur = cur->next;

    }

  xmlFreeDoc (doc);

  return 0;
}

void
search_write (char *who, struct post *tmp, int *i)
{
  char *t, *k;
  int len;
  char output[SIZE_BUFFER];

  (*i)++;

#ifdef HAVE_LIBRECODE
  t = recode (tmp->date);
  k = recode (tmp->author);
#else
  t = tmp->date;
  k = tmp->author;
#endif

  len =
    snprintf (output, SIZE_BUFFER,
	      "PRIVMSG %s :%2d - Id: %d - date: %s - author: %s\n", who, *i,
	      tmp->id, t, k);
  irc_write (output, len);

#ifdef HAVE_LIBRECODE
  free (t);
  free (k);
#endif

  if (tmp->url)
    {

#ifdef HAVE_LIBRECODE
      t = recode (tmp->url);
#else
      t = tmp->url;
#endif

      len =
	snprintf (output, SIZE_BUFFER, "PRIVMSG %s :     url: %s\n", who, t);
      irc_write (output, len);

#ifdef HAVE_LIBRECODE
      free (t);
#endif

    }

  if (tmp->title)
    {

#ifdef HAVE_LIBRECODE
      t = recode (tmp->title);
#else
      t = tmp->title;
#endif

      len =
	snprintf (output, SIZE_BUFFER, "PRIVMSG %s :     title: %s\n", who,
		  t);
      irc_write (output, len);

#ifdef HAVE_LIBRECODE
      free (t);
#endif

    }
}
