/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

void
irc_socket (void)
{
  struct sockaddr_in sock;
  struct hostent *hp;
  int len;
  char buff[SIZE_BUFFER];
  pthread_t th;
#ifdef HAVE_LIBSSL
  SSL_CTX *ctx;
  SSL *ssl;
#endif

#ifdef HAVE_LIBSSL
  if (secure)
    {
      char *tmp;

      SSLeay_add_ssl_algorithms ();
      ctx = SSL_CTX_new (SSLv23_client_method ());
      bio = BIO_new_ssl_connect (ctx);
      BIO_get_ssl (bio, &ssl);

      if (!ssl)
	fatal ("Error: ssl connection.");

      SSL_set_mode (ssl, SSL_MODE_AUTO_RETRY);

      if (!(tmp = (char *) malloc ((strlen (server) + 7) * sizeof (char))))
	fatal ("Error: memory.");

      sprintf (tmp, "%s:%d", server, port);

      if (BIO_set_conn_hostname (bio, tmp) < 0)
	{
	  fprintf (stderr, "Error: ssl connection.\n");
	  BIO_ssl_shutdown (bio);
	  BIO_free_all (bio);
	  return;
	}

      if (BIO_do_connect (bio) < 0)
	{
	  fprintf (stderr, "Error: ssl connection.\n");
	  BIO_ssl_shutdown (bio);
	  BIO_free_all (bio);
	  return;
	}

      if (BIO_do_handshake (bio) < 0)
	{
	  fprintf (stderr, "Error: ssl connection.\n");
	  BIO_ssl_shutdown (bio);
	  BIO_free_all (bio);
	  return;
	}

      free (tmp);

    }
  else
#endif
    {

      if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
	{
	  fprintf (stderr, "Error: socket.\n");
	  return;
	}

      if (!(hp = gethostbyname (server)))
	{
	  close (fd);
	  fprintf (stderr, "Error: resolve %s.\n", server);
	  return;
	}

      memset ((void *) &sock, 0, sizeof (sock));
      sock.sin_family = AF_INET;
      sock.sin_addr.s_addr = ((struct in_addr *) (hp->h_addr))->s_addr;
      sock.sin_port = htons (port);

      if (connect (fd, (struct sockaddr *) &sock, sizeof (sock)) < 0)
	{
	  close (fd);
	  fprintf (stderr, "Error: connect to: %s.\n", server);
	  return;
	}

    }

  set_nick ();

  set_channel ();

  if (pthread_mutex_init (&mutex, NULL))
    fatal ("Error: mutex.\n");

  if (pthread_create (&th, NULL, thread_xml, NULL))
    fatal ("Error: pthread.\n");

  while ((len = get_line (buff)))
    {

      buff[len-1] = 0;

      if (debug)
	fprintf (stderr, "%s\n", buff);

      if (!strncmp (buff, "PING ", 5))
	{
	  buff[1] = 'O';
	  irc_write (buff, len);
	}
      else
	{
	  char *cmd;
	  int flag = 0;

	  cmd = check_cmd (buff);

	  if (cmd)
	    {
	      switch (atoi (cmd))
		{
		case 409:	/* ERR_NOORIGIN */
		case 431:	/* ERR_NONICKNAMEGIVEN */
		case 432:	/* ERR_ERRONEUSNICKNAME */
		case 433:	/* ERR_NICKNAMEINUSE */
		case 436:	/* ERR_NICKCOLLISION */
		  set_nick ();
		  set_channel ();
		  flag = 1;
		  break;

		case 471:	/* ERR_CHANNELISFULL */
		case 474:	/* ERR_BANNEDFROMCHAN */
		  set_channel ();
		  flag = 1;
		  break;
		}

	      if (!flag)
		{
		  if (!strcmp (cmd, "KICK") && len > strlen (cmd))
		    {
		      char *who;

		      who = check_part (cmd + strlen (cmd) + 1);

		      if (!strcmp (who, nick))
			set_channel ();
		    }

		  else if (!strcmp (cmd, "PRIVMSG") && len > strlen (cmd))
		    {
		      char *who, *what;

		      who = check_part (cmd + strlen (cmd) + 1);

		      if (len <= strlen (cmd) + strlen (who) + 1)
			continue;

		      what = who + strlen (who) + 1;

		      if (!strcmp (who, nick))
			{
			  char *t;

			  if (*buff == ':')
			    who = buff + 1;
			  else
			    who = buff;

			  t = buff;

			  while (*t && *t != '!' && *t != ' ' && *t != '\t')
			    t++;
			  *t = 0;

			  query_check (who, what);
			}
		    }
		}
	    }
	}
    }

#ifdef HAVE_LIBSSL
  if (secure)
    {
      BIO_ssl_shutdown (bio);
      BIO_free_all (bio);
    }
  else
#endif
    close (fd);

  pthread_join (th, NULL);

  pthread_mutex_destroy (&mutex);

}

void
set_nick (void)
{
  char buff[SIZE_BUFFER];
  int len;

  len = snprintf (buff, SIZE_BUFFER, "NICK %s\n", nick);
  irc_write (buff, len);

  len =
    snprintf (buff, SIZE_BUFFER, "USER %s %s %s %s\n", nick, nick, nick,
	      nick);
  irc_write (buff, len);

  if (password)
    {
      len =
	snprintf (buff, SIZE_BUFFER, "PRIVMSG nickserv :identify %s\n",
		  password);
      irc_write (buff, len);
    }
}

void
set_channel (void)
{
  char buff[SIZE_BUFFER];
  struct channel *tmp;
  int len;

  tmp = channels;
  while (tmp)
    {

      len = snprintf (buff, SIZE_BUFFER, "PART %s\n", tmp->channel);
      irc_write (buff, len);

      len = snprintf (buff, SIZE_BUFFER, "JOIN %s\n", tmp->channel);
      irc_write (buff, len);

      tmp = tmp->next;
    }
}
