/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

void
check (struct channel *chan)
{
  struct MemoryStruct *chunk;

  if (debug)
    fprintf (stderr, "Downloading file '%s'...\n", chan->file);

  check_time = 0;
  check_write = 0;

  if (!(chunk = download_file (chan->file)))
      return;

  parse (chunk->mm, chunk->size, chan);

  free (chunk->mm);
  free (chunk);
}

void
parseStory (xmlDocPtr doc, xmlNodePtr cur, struct channel *chan)
{

  struct newswire nw;
  static char tmp[SIZE_BUFFER];
  int len;

  cur = cur->xmlChildrenNode;

  if (cur && cur->next)
    cur = cur->next;
  else
    return;

  nw.title = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (cur && cur->next && cur->next->next)
    cur = cur->next->next;
  else
    return;

  nw.url = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (cur && cur->next && cur->next->next)
    cur = cur->next->next;
  else
    return;

  nw.time = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (cur && cur->next && cur->next->next)
    cur = cur->next->next;
  else
    return;

  nw.author = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (cur && cur->next && cur->next->next)
    cur = cur->next->next;
  else
    return;

  nw.department = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (cur && cur->next && cur->next->next)
    cur = cur->next->next;
  else
    return;

  nw.topic = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (cur && cur->next && cur->next->next)
    cur = cur->next->next;
  else
    return;

  nw.comments = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (cur && cur->next && cur->next->next)
    cur = cur->next->next;
  else
    return;

  nw.section = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (cur && cur->next && cur->next->next)
    cur = cur->next->next;
  else
    return;

  nw.image = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);

  if (check_time < MAX_NEWS && !chan->old_id[check_time])
    {

      if (debug)
	fprintf (stderr, "Cache empty: add ID %d.\n", get_id (nw.url));

      chan->new_id[check_time] = get_id (nw.url);

      check_time++;
      if (check_time == MAX_NEWS)
	check_time = -1;

      check_write = 1;
    }

  else
    {
      if (check_time < MAX_NEWS)
	{
	  chan->new_id[check_time] = get_id (nw.url);

	  check_time++;
	}

      for (len = 0; !check_write && len < MAX_NEWS; len++)
	{
	  if (chan->old_id[len] == get_id (nw.url))
	    check_write = 1;

	  if (debug)
	    fprintf (stderr, "Check ID %d with cache %d\n",
		     get_id (nw.url), chan->old_id[len]);

	}

      if (!check_write)
	{
	  char *url, *title, *author;

#ifdef HAVE_LIBRECODE
	  url = recode (nw.url);
	  title = recode (trim (nw.title));
	  author = recode (trim (nw.author));
#else
	  url = nw.url;
	  title = trim (nw.title);
	  author = trim (nw.author);
#endif

	  len =
	    snprintf (tmp, SIZE_BUFFER,
		      "PRIVMSG %s :(%d) - %s - by %s - %s\n", chan->channel,
		      get_id (nw.url), title, author, url);

#ifdef HAVE_LIBRECODE
	  free (url);
	  free (title);
	  free (author);
#endif

	  if (irc_write (tmp, len) != len)
	    {
	      return;
	    }
	}
    }

  xmlFree (nw.title);
  xmlFree (nw.url);
  xmlFree (nw.time);
  xmlFree (nw.author);
  xmlFree (nw.department);
  xmlFree (nw.topic);
  xmlFree (nw.comments);
  xmlFree (nw.section);
  xmlFree (nw.image);

  return;
}

void
parse (void *mm, size_t size, struct channel *chan)
{

  xmlDocPtr doc;
  xmlNodePtr cur;
  register int i;

  doc = xmlParseMemory (mm, size);

  if (doc == NULL)
    {
      fprintf (stderr, "Error: parse xml file.\n");
      return;
    }

  cur = xmlDocGetRootElement (doc);

  if (cur == NULL)
    {
      fprintf (stderr, "Error: no info in xml file.\n");
      return;
    }

  if (xmlStrcmp (cur->name, (const xmlChar *) "backslash"))
    {
      fprintf (stderr, "Error: parse xml file.\n");
      return;
    }

  cur = cur->xmlChildrenNode;
  if (cur)
    cur = cur->next;

  if (debug)
    fprintf (stderr, "Check xml structs...\n");

  while (cur != NULL)
    {
      if ((!xmlStrcmp (cur->name, (const xmlChar *) "story")))
	parseStory (doc, cur, chan);

      if (check_time == -1)
	break;

      cur = cur->next;
    }

  for (i = 0; i < MAX_NEWS; i++)
    {
      chan->old_id[i] = chan->new_id[i];

      chan->new_id[i] = 0;
    }


  if (debug)
    fprintf (stderr, "Parsing OK\n");

  xmlFreeDoc (doc);
  return;
}
